// hlavickove subory *******************************************************
#include "../includes.h"
// vlastny .h subor
#include "fbo.h"

// konstruktor *************************************************************
CFBO::CFBO()
{
 created=-1; // FBO neexistuje
}

// destruktor **************************************************************
CFBO::~CFBO()
{
 cancel(); // zrusenie FBO
}

// inicializacia stvoruholnika predstavujuceho FBO zlozeny z 2 trojuholnikov
void CFBO::init_obj(void) {
 // stvoruholnik predstavujuci FBO zlozeny z 2 trojuholnikov
 loopi(18) v[i]=0.0f;
 v[0]=-1.0f; v[1]=-1.0f; v[3]=1.0f; v[4]=-1.0f; v[6]=1.0f; v[7]=1.0f;
 v[9]=-1.0f; v[10]=-1.0f; v[12]=1.0f; v[13]=1.0f; v[15]=-1.0f; v[16]=1.0f;
 t[0]=0.0f; t[1]=0.0f; t[2]=1.0f; t[3]=0.0f; t[4]=1.0f; t[5]=1.0f;
 t[6]=0.0f; t[7]=0.0f; t[8]=1.0f; t[9]=1.0f; t[10]=0.0f; t[11]=1.0f;
}

// inicializacia FBO0 ******************************************************
int CFBO::init0(void)
{
 // FBO0 pre scr_w, scr_h
 glGenFramebuffersEXT(1,&f0.id);
 glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,f0.id);

 // 1 depth
 glGenRenderbuffersEXT(1,&f0.depth);
 glBindRenderbufferEXT(GL_RENDERBUFFER_EXT,f0.depth);
 glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT,GL_DEPTH_COMPONENT,scr_w,scr_h);
 glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT,GL_DEPTH_ATTACHMENT_EXT,GL_RENDERBUFFER_EXT,f0.depth);
 glBindRenderbufferEXT(GL_RENDERBUFFER_EXT,0);

 // 2 frame
 loopi(2) {
  glGenTextures(1,&(f0.tex_id[i]));
  glBindTexture(GL_TEXTURE_2D,f0.tex_id[i]);

  glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA16F_ARB,scr_w,scr_h,0,GL_RGBA,GL_FLOAT,NULL);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
  switch (i) {
   case 0: glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_2D,f0.tex_id[i],0); break;
   case 1: glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT1_EXT,GL_TEXTURE_2D,f0.tex_id[i],0); break;
  }

  printf("FBO0 frame textura: %u\n",f0.tex_id[i]);
 }

 ret=glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
 if (ret!=GL_FRAMEBUFFER_COMPLETE_EXT) {
  printf("FBO0 error: 0x%x\n",ret); return(1);
 }

 // aplikaciou alokovany framebuffer
 glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,0);

 init_obj(); // inicializacia stvoruholnika

 created=0; // FBO0 bol uspesne vytvoreny

 return(0);
}

// inicializacia FBO1 a FBO2 ***********************************************
int CFBO::init(void)
{
 // FBO1 pre scr_w, scr_h
 glGenFramebuffersEXT(1,&f1.id);
 glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,f1.id);

 // 1 depth
 glGenRenderbuffersEXT(1,&f1.depth);
 glBindRenderbufferEXT(GL_RENDERBUFFER_EXT,f1.depth);
 glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT,GL_DEPTH_COMPONENT,scr_w,scr_h);
 glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT,GL_DEPTH_ATTACHMENT_EXT,GL_RENDERBUFFER_EXT,f1.depth);
 glBindRenderbufferEXT(GL_RENDERBUFFER_EXT,0);
 
 // 1 frame
 glGenTextures(1,&(f1.tex_id));
 glBindTexture(GL_TEXTURE_2D,f1.tex_id);

 glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA16F_ARB,scr_w,scr_h,0,GL_RGBA,GL_FLOAT,NULL);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_2D,f1.tex_id,0);

 ret=glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
 if (ret!=GL_FRAMEBUFFER_COMPLETE_EXT) {
  printf("FBO1 error: 0x%x\n",ret); return(1);
 }

 printf("FBO1 frame textura: %u\n",f1.tex_id);

 // FBO2 pre scr_w2, scr_h2
 glGenFramebuffersEXT(1,&f2.id);
 glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,f2.id);

 // 0 depth
 // 3 frame
 loopi(3) {
  glGenTextures(1,&(f2.tex_id[i]));
  glBindTexture(GL_TEXTURE_2D,f2.tex_id[i]);

  glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA16F_ARB,scr_w2,scr_h2,0,GL_RGBA,GL_FLOAT,NULL);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
  switch (i) {
   case 0: glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_2D,f2.tex_id[i],0); break;
   case 1: glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT1_EXT,GL_TEXTURE_2D,f2.tex_id[i],0); break;
   case 2: glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT2_EXT,GL_TEXTURE_2D,f2.tex_id[i],0); break;
  }

  printf("FBO2 frame textura: %u\n",f2.tex_id[i]);
 }

 ret=glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
 if (ret!=GL_FRAMEBUFFER_COMPLETE_EXT) {
  printf("FBO2 error: 0x%x\n",ret); return(1);
 }

 // aplikaciou alokovany framebuffer
 glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,0);

 init_obj(); // inicializacia stvoruholnika

 created=1; // FBO1 a FBO2 boli uspesne vytvorene

 return(0);
}

// inicializacia FBO pre downsampler ***************************************
int CFBO::init_downsampler(void)
{
 int w=scr_w,h=scr_h; // originalna velkost okna

 // 4 FBO pre downsampler kazdy dalsi s polovicnou velkostou
 loopi(4) {
  w/=2; h/=2; // polovicna velkost okna

  // frame buffer object
  glGenFramebuffersEXT(1,&(f_downsampler[i].id));
  glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,f_downsampler[i].id);

  // depth
  glGenRenderbuffersEXT(1,&(f_downsampler[i].depth));
  glBindRenderbufferEXT(GL_RENDERBUFFER_EXT,f_downsampler[i].depth);
  glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT,GL_DEPTH_COMPONENT,w,h);
  glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT,GL_DEPTH_ATTACHMENT_EXT,GL_RENDERBUFFER_EXT,f_downsampler[i].depth);
  glBindRenderbufferEXT(GL_RENDERBUFFER_EXT,0);

  // frame
  glGenTextures(1,&(f_downsampler[i].tex_id));
  glBindTexture(GL_TEXTURE_2D,f_downsampler[i].tex_id);

  glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA16F_ARB,w,h,0,GL_RGBA,GL_FLOAT,NULL);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
  glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT,GL_COLOR_ATTACHMENT0_EXT,GL_TEXTURE_2D,f_downsampler[i].tex_id,0);

  ret=glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
  if (ret!=GL_FRAMEBUFFER_COMPLETE_EXT) {
   printf("FBO for downsampler error: 0x%x\n",ret); return(1);
  }

  printf("FBO pre downsampler frame textura %ix%i: %u\n",w,h,f_downsampler[i].tex_id);
 }

 // aplikaciou alokovany framebuffer
 glBindFramebufferEXT(GL_FRAMEBUFFER_EXT,0);

 init_obj(); // inicializacia stvoruholnika

 created=2; // FBO pre downsampler bol uspesne vytvoreny

 return(0);
}

// zrusenie FBO ************************************************************
void CFBO::cancel(void)
{
 switch (created) {
  case 0: // FBO0
          glDeleteRenderbuffersEXT(1,&f0.depth);
          loopi(2) {
           glDeleteTextures(1,&(f0.tex_id[i]));
           printf("zrusenie FBO0 frame textury: %u\n",f0.tex_id[i]);
          }
          glDeleteFramebuffersEXT(1,&f0.id);
          break;
  case 1: // FBO1
          glDeleteTextures(1,&(f1.tex_id));
          glDeleteRenderbuffersEXT(1,&f1.depth);
          glDeleteFramebuffersEXT(1,&f1.id);
          printf("zrusenie FBO1 frame textury: %u\n",f1.tex_id);

          // FBO2
          loopi(3) {
           glDeleteTextures(1,&(f2.tex_id[i]));
           printf("zrusenie FBO2 frame textury: %u\n",f2.tex_id[i]);
          }
          glDeleteFramebuffersEXT(1,&f2.id);
          break;
  case 2: // 4 FBO pre downsampler
          loopi(4) {
           glDeleteRenderbuffersEXT(1,&(f_downsampler[i].depth));
           glDeleteTextures(1,&(f_downsampler[i].tex_id));
           printf("zrusenie FBO pre downsampler frame textury: %u\n",f_downsampler[i].tex_id);
           glDeleteFramebuffersEXT(1,&(f_downsampler[i].id));
          }
          break;
  default: break;
 }
}

// vykreslenie obsahu FBO1 *************************************************
void CFBO::drawfbo1_attachment(void)
{
 glBindTexture(GL_TEXTURE_2D,f1.tex_id);

 // vrcholy
 glVertexAttribPointer(0,3,GL_FLOAT,GL_FALSE,0,v); // ukazovatel na udaje
 // texturovacie suradnice
 glVertexAttribPointer(8,2,GL_FLOAT,GL_FALSE,0,t);

 glDrawArrays(GL_TRIANGLES,0,6);
}

// vykreslenie obsahu FBO2 *************************************************
void CFBO::drawfbo2_attachment(int N)
{
 glBindTexture(GL_TEXTURE_2D,f2.tex_id[N]);

 // vrcholy
 glVertexAttribPointer(0,3,GL_FLOAT,GL_FALSE,0,v); // ukazovatel na udaje
 // texturovacie suradnice
 glVertexAttribPointer(8,2,GL_FLOAT,GL_FALSE,0,t);

 glDrawArrays(GL_TRIANGLES,0,6);
}

// vykreslenie objektu FBO *************************************************
void CFBO::draw(void)
{
 // vrcholy
 glVertexAttribPointer(0,3,GL_FLOAT,GL_FALSE,0,v); // ukazovatel na udaje
 // texturovacie suradnice
 glVertexAttribPointer(8,2,GL_FLOAT,GL_FALSE,0,t);

 glDrawArrays(GL_TRIANGLES,0,6);
}
