// hlavickove subory *******************************************************
#include "../includes.h"
// vlastny .h subor
#include "image.h"

#include "rgbe.h" // pre nacitanie obsahu HDR suboru

// konstruktor *************************************************************
CImage::CImage()
{
 data=NULL;      // udaje obrazku
 width=height=0; // sirka a vyska obrazku
}

// destruktor **************************************************************
CImage::~CImage()
{
 DELETEA(data); // uvolnenie pamate obsadenej udajmi obrazku
}

// nacitanie udajov obrazku zo suboru **************************************
int CImage::loadFromBmp(const char *F)
{
 // otvorenie suboru
 FILE *f_p=NULL;
 f_p=fopen(F,"rb");
 if (f_p==NULL) { printf("Nemozno otvorit subor: %s!\n",F); return(1); }

 // sirka a vyska obrazku
 fseek(f_p,18,SEEK_CUR);
 res=fread(&width,4,1,f_p); res=fread(&height,4,1,f_p);

 // velkost udajov obrazku v pripade formatu BGR
 unsigned int s=width*height*3; // 3 udaje na 1 pixel - BGR

 // pocet vrstiev
 unsigned short int layers; res=fread(&layers,2,1,f_p);
 if (layers!=1) { printf("Nepodporovany format suboru: %s!\n",F); fclose(f_p); return(1); }

 // pocet bitov na pixel
 unsigned short int bpp; res=fread(&bpp,2,1,f_p);
 if (bpp!=24) { printf("Nepodporovany format suboru: %s!\n",F); fclose(f_p); return(1); }

 // pridelenie pamate pre ulozenie udajov obrazku
 data=new char[s];
 char *tempData=new char[s];

 // nacitanie BGR udajov
 fseek(f_p,24,SEEK_CUR);
 res=fread(tempData,s,1,f_p);

 // konverzia BGR na RGB
 int j=0;
 loopi(s/3) { 
  data[j+0]=tempData[j+2]; data[j+1]=tempData[j+1]; data[j+2]=tempData[j+0];
  j+=3;
 }

 // uvolnenie docasne prideleneho pamatoveho miesta
 DELETEA(tempData);

 // zatvorenie suboru
 fclose(f_p);

 // OpenGL format a typ
 format=GL_RGB; type=GL_UNSIGNED_BYTE;

 return(0);
}

// nacitanie udajov obrazku zo suboru **************************************
int CImage::loadFromHdr(const char *F)
{
 // otvorenie suboru
 FILE *f_p=NULL;
 f_p=fopen(F,"rb");
 if (f_p==NULL) { printf("Nemozno otvorit subor: %s!\n",F); return(1); }

 // nacitanie RGBE hlavicky
 int res=RGBE_ReadHeader(f_p,&width,&height,0);
 if (res<0) { printf("Nepodporovany format suboru: %s!\n",F); fclose(f_p); return(1); }

 // pridelenie pamate pre ulozenie udajov obrazku
 data=new char[sizeof(float)*3*width*height];
 // nacitanie udajov
 res=RGBE_ReadPixels_RLE(f_p,reinterpret_cast<float*>(data),width,height);

 // zatvorenie suboru
 fclose(f_p);

 // OpenGL format a typ
 format=GL_RGB; type=GL_FLOAT;

 return(0);
}

// pocet Bytov na 1 pixel obrazku ******************************************
int CImage::getBpp() const
{
 int componentNum=0; // pocet udajov na 1 pixel
 switch (format) {
  case GL_RGBA: componentNum=4; break;
  case GL_RGB: componentNum=3; break;
  default: break;
 }
 int componentSize=0; // pocet B na 1 udaj
 switch (type) {
  case GL_UNSIGNED_BYTE: componentSize=sizeof(char); break;
  case GL_FLOAT: componentSize=sizeof(float); break;
  default: break;
 }
 // bytes per pixel = pocet udajov * pocet B na 1 udaj
 return(componentNum*componentSize);
}

// otocenie obrazku ********************************************************
void CImage::flipHorizontal(void)
{
 const int stride=getBpp();
 const int w=width; const int h=height;
 const int m=h/2;
 char *temp=new char[stride];

 for (int y=0;y<m;++y) {
  for (int x=0;x<w;++x) {
   int offset=(y*w+x)*stride;
   int offsetOpp=((h-y-1)*w+x)*stride;
   memcpy(temp,data+offset,stride);
   memcpy(data+offset,data+offsetOpp,stride);
   memcpy(data+offsetOpp,temp,stride);
  }
 }
 DELETEA(temp);
}
void CImage::flipVertical(void)
{
 const int stride=getBpp();
 const int w=width; const int h=height;
 const int m=w/2;
 char *temp=new char[stride];

 for (int y=0;y<h;++y) {
  for (int x=0;x<m;++x) {
   int offset=(y*w+x)*stride;
   int offsetOpp=(y*w+(w-x-1))*stride;
   memcpy(temp,data+offset,stride);
   memcpy(data+offset,data+offsetOpp,stride);
   memcpy(data+offsetOpp,temp,stride);
  }
 }
 DELETEA(temp);
}
