// hlavickove subory *******************************************************
#include "../includes.h"
// vlastny .h subor
#include "texture.h"

#include "image.h" // pre nacitanie BMP a HDR suborov

// konstruktor *************************************************************
CTexture::CTexture()
{
 loopi(MAX_TEX_NUM) tex_id[i]=0;

 // pocet nahratych textur
 tex_count=0;

 // SDL nacitanie suboru s obrazkom textury
 img=NULL;
 // pomocna pamat
 data=NULL;
}

// destruktor **************************************************************
CTexture::~CTexture()
{
 // zrusenie objektov textur
 loopi(tex_count) {
  if (tex_id[i]!=0) freeTexture(i);
  else tex_count++; // ak objekt textury neexistuje, cyklus pokracuje
                    // na dalsom indexe az do zrusenia zadaneho poctu objektov
 }
 tex_count=0;

 DELETEA(clrs);
}

// uprava farieb bodov textury na zaklade typu suboru **********************
int CTexture::correctData(SDL_Surface *S)
{
 GLuint w=(GLuint) S->w,h=(GLuint) S->h;
 GLubyte *pixels=(GLubyte *) S->pixels;
 GLubyte palIndex;

 if (S->format==NULL) { printf("Chyba formatu obrazku!\n"); return(1); }

 // obrazok bez palety
 if (S->format->palette==NULL) {
  bpp=S->format->BytesPerPixel; // pocet bytov na texel
  // INFO: bpp
  //printf("bpp: %u\n",bpp);

  // vyhradenie pamate pre farby bodov textury
  data=new GLubyte[w*h*bpp];
  if (data==NULL) { printf("Nemozno vyhradit pamat!\n"); return(1); }

  unsigned int rr,gg,bb,aa;
  // kopirovanie R G B (A) zloziek farby
  switch (bpp) {
   case 3: if (S->format->Bshift!=0) { rr=0; bb=2; }
           else { rr=2; bb=0; }
           loopj(h) {
            for (unsigned int i=0;i<w*3;i+=3) {
             data[i+j*w*bpp+0]=pixels[i+j*w*bpp+rr]; // R - R/B
             data[i+j*w*bpp+1]=pixels[i+j*w*bpp+1];  // G - G
             data[i+j*w*bpp+2]=pixels[i+j*w*bpp+bb]; // B - B/R
            }
           }
           break;
   case 4: if (S->format->Ashift!=0) { rr=0; gg=1; bb=2; aa=3; }
           else { rr=3; gg=2; bb=1; aa=0; }
           loopj(h) {
            for (unsigned int i=0;i<w*4;i+=4) {
             data[i+j*w*bpp+0]=pixels[i+j*w*bpp+rr]; // R - R/A
             data[i+j*w*bpp+1]=pixels[i+j*w*bpp+gg]; // G - G/B
             data[i+j*w*bpp+2]=pixels[i+j*w*bpp+bb]; // B - B/G
             data[i+j*w*bpp+3]=pixels[i+j*w*bpp+aa]; // A - A/R
            }
           }
           break;
   default: printf("Nemozno skopirovat farby!\n"); DELETEA(data); return(1);
  }
 }
 // obrazok s paletou
 else {
  bpp=3; // predpoklad 3 byty na texel
  // INFO: bpp
  //printf("bpp: %u\n",bpp);

  // kontrola palety
  if (S->format->palette->colors==NULL) { printf("Chyba v palete obrazku!\n"); return(1); }

  // vyhradenie pamate pre farby bodov textury
  data=new GLubyte[w*h*bpp];
  if (data==NULL) { printf("Nemozno vyhradit pamat!\n"); return(1); }

  unsigned int k;
  // kopirovanie R G B zloziek farby
  loopj(h) {
   k=0;
   loopi(w) {
    palIndex=pixels[i+j*w]; // index do palety
    data[k+j*w*bpp+0]=S->format->palette->colors[palIndex].r; // R - R
    data[k+j*w*bpp+1]=S->format->palette->colors[palIndex].g; // G - G
    data[k+j*w*bpp+2]=S->format->palette->colors[palIndex].b; // B - B
    k+=3;
   }
  }
 }

 // vyber formatu udajov ulozenych v texture
 switch (bpp) {
  case 1: glFormat=GL_ALPHA; break;
  case 3: glFormat=GL_RGB; break;
  case 4: glFormat=GL_RGBA; break;
  default: printf("Nemozno vybrat format udajov ulozenych v texture!\n");
           DELETEA(data); return(1);
 }

 // vyber interneho formatu udajov ulozenych v texture
 switch (glFormat) {
  case GL_ALPHA:
   switch (bpp) {
    case 1: intFormat=GL_ALPHA; break;
    default: printf("Nemozno vybrat interny format udajov ulozenych v texture!\n");
             DELETEA(data); return(1);
   }
  break;
  case GL_RGB:
   switch (bpp) {
    case 3: intFormat=GL_RGB; break;
    default: printf("Nemozno vybrat interny format udajov ulozenych v texture!\n");
             DELETEA(data); return(1);
   }
  break;
  case GL_RGBA:
   switch (bpp) {
    case 4: intFormat=GL_RGBA4; break;
    default: printf("Nemozno vybrat interny format udajov ulozenych v texture!\n");
             DELETEA(data); return(1);
   }
  break;
 }

 return(0);
}

// nastavenie parametrov a vytvorenie 2D textury ***************************
void CTexture::genTexture(GLuint W,GLuint H,GLubyte *DATA,GLuint MODE)
{
 // nastavenie filteringu
 if (MODE & T_LINEAR) { // LINEAR
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  if (MODE & T_MIPMAP) {
   glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
  } else {
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    }
 } else {             // NEAREST
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
    if (MODE & T_MIPMAP) {
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_NEAREST);
    } else {
       glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
      }
   }
 // nastavenie opakovania sa textury
 if (MODE & T_CLAMP) {
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
 } else if (MODE & T_CLAMP_TO_EDGE) {
         glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
         glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
        } else {
           glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
           glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
          }
 // vytvorenie objektu textury
 if (MODE & T_MIPMAP) { // MIPMAP
  gluBuild2DMipmaps(GL_TEXTURE_2D,intFormat,W,H,glFormat,GL_UNSIGNED_BYTE,DATA);
 } else {             // standardna 2D textura
    glTexImage2D(GL_TEXTURE_2D,0,intFormat,W,H,0,glFormat,GL_UNSIGNED_BYTE,DATA);
   }
}

// nahratie 2D textury *****************************************************
int CTexture::load2D(GLuint ID,const char *F,GLuint MODE)
{
 if ((tex_count+1)>=MAX_TEX_NUM) {
  printf("Prekroceny maximalny pocet textur!\n"); return(1);
 }

 // nacitanie suboru s obrazkom textury
 img=NULL; img=IMG_Load(F); if (img==NULL) return(1);

 // uprava farieb bodov textury na zaklade typu suboru
 if (correctData(img)) {
  printf("Nemozno upravit farby bodov textury!\n");
  // uvolnenie pouzitej pamate
  if (img) SDL_FreeSurface(img); img=NULL;
  return(1);
 }

 // vygenerovanie textury
 glGenTextures(1,&tex_id[ID]);
 tex_count++; // zvysenie poctu textur
 // pripojenie textury
 glBindTexture(GL_TEXTURE_2D,tex_id[ID]);
 // nastavenie parametrov a vytvorenie 2D textury
 genTexture((GLuint) img->w,(GLuint) img->h,data,MODE);

 // uvolnenie vyhradenej pamate (vyhradenej v correctData())
 DELETEA(data);

 // uvolnenie pouzitej pamate
 if (img) SDL_FreeSurface(img); img=NULL;

 // INFO: identifikator a nazov suboru textury
 printf("2D textura: %u %s\n",tex_id[ID],F);

 return(0);
}

// vytvorenie grayscale 2D textury *****************************************
int CTexture::create_grayscale(GLuint ID)
{
 if ((tex_count+1)>=MAX_TEX_NUM) {
  printf("Prekroceny maximalny pocet textur!\n"); return(1);
 }

 // vyhradenie pamate pre odtiene bodov textury
 clrs=NULL;
 clrs=new char[512*512];
 if (clrs==NULL) { printf("Nemozno vyhradit pamat!\n"); return(1); }

 // vygenerovanie textury
 glGenTextures(1,&tex_id[ID]);
 tex_count++; // zvysenie poctu textur
 // pripojenie textury
 glBindTexture(GL_TEXTURE_2D,tex_id[ID]);
 // nastavenie parametrov a vytvorenie 2D textury
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
 glTexImage2D(GL_TEXTURE_2D,0,GL_ALPHA,512,512,0,GL_ALPHA,GL_UNSIGNED_BYTE,clrs);

 // INFO: identifikator a nazov suboru textury
 printf("2D grayscale textura: %u\n",tex_id[ID]);

 return(0);
}

#define PERLIN_NOISE_TEX_SIZE 64
#define OCTAVES 4
#define PERSISTENCE 0.5f

// priprava 3D textury *****************************************************
int CTexture::prepare_perlin_noise_tex(GLuint ID)
{
 if ((tex_count+1)>=MAX_TEX_NUM) {
  printf("Prekroceny maximalny pocet textur!\n"); return(1);
 }

 // vygenerovanie objektu textury
 glGenTextures(1,&tex_id[ID]);
 tex_count++; // zvysenie poctu textur
 // pripojenie objektu textury
 glBindTexture(GL_TEXTURE_3D,tex_id[ID]);

 // vyhradenie pamate pre 3D texturu
 GLubyte *ptr=NULL;
 ptr=(GLubyte *) malloc(PERLIN_NOISE_TEX_SIZE*PERLIN_NOISE_TEX_SIZE*PERLIN_NOISE_TEX_SIZE);
 if (ptr==NULL) return(1);

 printf("Priprava 3D perlin noise textury...\n");
 // vytvorenie perlin noise 3D textury
 unsigned int p=0;
 loopk(PERLIN_NOISE_TEX_SIZE) {   // v smere Z
  loopj(PERLIN_NOISE_TEX_SIZE) {  // v smere Y
   loopi(PERLIN_NOISE_TEX_SIZE) { // v smere X
    // farba = 128 + 128.0f * perlin_noise
    // hodnota perlin_noise je v intervale <-a;a> v zavislosti od amplitudy
    *(ptr+p)=128+(GLubyte) (128.0f*perlin_noise(i,j,k));
    p++;
   }
  }
 }
 printf("ok\n");

 // opakovanie textury vo vsetkych smeroch
 glTexParameterf(GL_TEXTURE_3D,GL_TEXTURE_WRAP_S,GL_REPEAT);
 glTexParameterf(GL_TEXTURE_3D,GL_TEXTURE_WRAP_T,GL_REPEAT);
 glTexParameterf(GL_TEXTURE_3D,GL_TEXTURE_WRAP_R,GL_REPEAT);
 // linearne filtrovanie textury
 glTexParameterf(GL_TEXTURE_3D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameterf(GL_TEXTURE_3D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 // vytvorenie objektu 3D textury
 glTexImage3D(GL_TEXTURE_3D,0,GL_LUMINANCE,
              PERLIN_NOISE_TEX_SIZE,PERLIN_NOISE_TEX_SIZE,PERLIN_NOISE_TEX_SIZE,
              0, GL_LUMINANCE,GL_UNSIGNED_BYTE,ptr);
 
 // uvolnenie vyhradenej pamate
 free(ptr);

 // INFO: identifikator textury
 printf("3D textura: %u\n",tex_id[ID]);

 return(0);
}

// zrusenie textury ********************************************************
void CTexture::freeTexture(GLuint ID)
{
 printf("zrusenie textury id: %u\n",tex_id[ID]);
 glDeleteTextures(1,&tex_id[ID]);
}

// vyhladene pseudonahodne cislo *******************************************
inline float CTexture::smooth_randnum(int X,int Y,int Z)
{
 // Gaussovo vyhladenie pomocou 3D matice
 //     32    16    32
 //
 //   16    8    16
 //
 // 32   16   32
 //
 // 16   8    16
 //
 // 32   16   32
 // vaha vnutorneho prvku X,Y,Z = 4
 return((randnum(X-1,Y-1,Z-1)+randnum(X+1,Y-1,Z-1)+
         randnum(X+1,Y+1,Z-1)+randnum(X-1,Y+1,Z-1)+
         randnum(X-1,Y-1,Z+1)+randnum(X+1,Y-1,Z+1)+
         randnum(X+1,Y+1,Z+1)+randnum(X-1,Y+1,Z+1))/32.0f+
        (randnum(X-1,Y,  Z-1)+randnum(X+1,Y,  Z-1)+
         randnum(X+1,Y,  Z+1)+randnum(X-1,Y,  Z+1)+
         randnum(X,  Y-1,Z-1)+randnum(X,  Y-1,Z+1)+
         randnum(X,  Y+1,Z+1)+randnum(X,  Y+1,Z-1)+
         randnum(X-1,Y+1,Z  )+randnum(X-1,Y-1,Z  )+
         randnum(X+1,Y-1,Z  )+randnum(X+1,Y+1,Z  ))/16.0f+
        (randnum(X,  Y,  Z-1)+randnum(X+1,Y,  Z  )+
         randnum(X,  Y,  Z+1)+randnum(X-1,Y,  Z  )+
         randnum(X,Y+1,  Z  )+randnum(X,Y-1,  Z  ))/8.0f+
        (randnum(X,  Y,  Z  ))/4.0f);
}

// trilinearna = 3D interpolacia pseudonahodnych cisiel ********************
inline float CTexture::trilinear_randnum(float X,float Y,float Z)
{
 // 2D interpolacia
 //  ix,iy+1,iz+1 <--- i4 ---> ix+1,iy+1,iz+1
 //                   /
 // ix,iy+1,iz <--- i2 ---> ix+1,iy+1,iz
 //
 //  ix,iy,iz+1   <--- i3 ---> ix+1,iy,iz+1
 //                   /
 // ix,iy,iz   <--- i1 ---> ix+1,iy,iz
 ix=(int) X; iy=(int) Y; iz=(int) Z;
 i1=interpolate(smooth_randnum(ix,iy,  iz),  smooth_randnum(ix+1,iy,  iz),  X-ix);
 i2=interpolate(smooth_randnum(ix,iy+1,iz),  smooth_randnum(ix+1,iy+1,iz),  X-ix);
 i3=interpolate(smooth_randnum(ix,iy,  iz+1),smooth_randnum(ix+1,iy,  iz+1),X-ix);
 i4=interpolate(smooth_randnum(ix,iy+1,iz+1),smooth_randnum(ix+1,iy+1,iz+1),X-ix);
 i1=interpolate(i1,i3,Z-iz);
 i2=interpolate(i2,i4,Z-iz);
 // 2D interpolovane hodnoty ulozene v premennych i1 a i2 sa interpoluju podla Y
 return(interpolate(i1,i2,Y-iy));
}

// Perlinov sum ************************************************************
float CTexture::perlin_noise(float X,float Y,float Z)
{
 // vseobecny postup:
 // 1. vygenerovanie nahodnych (pseudonahodnych cisiel), dolezitou podmienkou je,
 //    aby bolo konkretnym suradniciam X,Y,Z pridelene vzdy rovnake nahodne cislo
 //    => 2 rovnake vstupy = rovnaky vystup
 // 2. pre kvalitnejsi vysledok pseudonahodne cisla vyhladime (smooth)
 // 3. interpolacia vygenerovanych pseudonahodnych cisiel
 // 4. spojenie viacerych funkcii sumu so zvysujucou sa frekvenciou a znizujucou sa amplitudou
 //    funkcia sumu danej frekvencie a amplitudy = oktava
 //    ak persistence (stalost) = 0.5, kazda dalsia oktava ma 2-nasobnu frekvenciu
 //    a kazda dalsia oktava ma 1/2-lovicnu amplitudu

 freq=0.2f; ampl=1.0f; col=0.0f;

 loopi(OCTAVES) {
  col+=trilinear_randnum(X*freq,Y*freq,Z*freq)*ampl;

  freq*=1.0f/PERSISTENCE; ampl*=PERSISTENCE;
 }

 return(col);
}

// nacitanie udajov obrazku ************************************************
CImage *loadImg(const char *F)
{
 // ziskanie pripony suboru
 char ext[4]; int j=strlen(F);
 loopi(j) if (F[i]=='.') { j=i+1; break; }
 ext[0]=F[j]; ext[1]=F[j+1]; ext[2]=F[j+2]; ext[3]='\0';

 // nacitanie obsahu suboru v zavislosti od jeho typu
 CImage *I=new CImage();
 j=0;
 if (!strcmp(ext,"bmp")) j=I->loadFromBmp(F);
 else if (!strcmp(ext,"hdr")) j=I->loadFromHdr(F);
      else { printf("Nepodporovany typ suboru!\n"); DELETEP(I); return(NULL); }

 // ak sa nepodarilo nacitat obsah suboru
 if (j!=0) { DELETEP(I); return(NULL); }

 return(I);
}

// nahratie CubeMap textury ************************************************
int CTexture::loadCubeMap(GLuint ID,const char *F)
{
 if ((tex_count+1)>=MAX_TEX_NUM) {
  printf("Prekroceny maximalny pocet textur!\n"); return(1);
 }

 CImage *cubeMap,*cubeFace[6],*img;

 // nacitanie suboru so 6-timi obrazkami
 cubeMap=NULL; cubeMap=loadImg(F); if (cubeMap==NULL) return(1);
 // sirka a vyska obrazku
 int w=cubeMap->getWidth(); int h=cubeMap->getHeight();
 // sirka a vyska casti obrazku (stena CubeMap)
 int fw=w/3; int fh=h/4;

 // vyber 6-tich obrazkov zo spolocneho suboru
 loopi(6) {
  cubeFace[i]=new CImage();                          // vytvorenie objektu
  //cubeFace[i]->data=new char[sizeof(float)*3*fw*fh]; // pridelenie pamate pre udaje
  cubeFace[i]->data=new char[sizeof(char)*3*fw*fh];  // pridelenie pamate pre udaje
  cubeFace[i]->format=GL_RGB;                        // zakladny format udajov
  //cubeFace[i]->type=GL_FLOAT;                        // zakladny typ udajov
  cubeFace[i]->type=GL_UNSIGNED_BYTE;                // zakladny typ udajov
  cubeFace[i]->width=fw;                             // sirka
  cubeFace[i]->height=fh;                            // vyska
 }

 // udaje obrazku ako ukazovatel na float (alebo char)
 //float *srcData=cubeMap->getDataFloat();
 char *srcData=cubeMap->getData();

 // kopia pamate zo spolocneho suboru do jednotlivych casti
 for (int y=0;y<h;++y) {
  for (int x=0;x<3;++x) {
   //int stride=fw*3*sizeof(float);
   int stride=fw*3*sizeof(char);
   int srcOffset=3*(y*w+x*fw);
   int offset=3*((y%fh)*fw);

   // CUBE_FACE_POSITIVE_X
   if (y>=fh && y<2*fh && x==2) memcpy(cubeFace[CUBE_FACE_POSITIVE_X]->getData()+offset,srcData+srcOffset,stride);
   // CUBE_FACE_NEGATIVE_X
   if (y>=fh && y<2*fh && x==0) memcpy(cubeFace[CUBE_FACE_NEGATIVE_X]->getData()+offset,srcData+srcOffset,stride);
   // CUBE_FACE_POSITIVE_Y
   if (y>=0 && y<fh && x==1) memcpy(cubeFace[CUBE_FACE_POSITIVE_Y]->getData()+offset,srcData+srcOffset,stride);
   // CUBE_FACE_NEGATIVE_Y
   if (y>=2*fh && y<3*fh && x==1) memcpy(cubeFace[CUBE_FACE_NEGATIVE_Y]->getData()+offset,srcData+srcOffset,stride);
   // CUBE_FACE_POSITIVE_Z
   if (y>=fh && y<2*fh && x==1) memcpy(cubeFace[CUBE_FACE_POSITIVE_Z]->getData()+offset,srcData+srcOffset,stride);
   // CUBE_FACE_NEGATIVE_Z
   if (y>=3*fh && y<4*fh && x==1) memcpy(cubeFace[CUBE_FACE_NEGATIVE_Z]->getData()+offset,srcData+srcOffset,stride);
  }
 }
 // CUBE_FACE_NEGATIVE_Z musi byt otocena
 cubeFace[CUBE_FACE_NEGATIVE_Z]->flipHorizontal();
 cubeFace[CUBE_FACE_NEGATIVE_Z]->flipVertical();

 // 6 klasickych textur ----------------------------------------------------
 loopi(6) {
  // vygenerovanie textury
  glGenTextures(1,&tex_id[ID+i]);
  tex_count++; // zvysenie poctu textur
  // pripojenie textury
  glBindTexture(GL_TEXTURE_2D,tex_id[ID+i]);

  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);

  switch (i) {
   case 0: img=cubeFace[CUBE_FACE_POSITIVE_X]; break;
   case 1: img=cubeFace[CUBE_FACE_NEGATIVE_X]; break;
   case 2: img=cubeFace[CUBE_FACE_POSITIVE_Y]; break;
   case 3: img=cubeFace[CUBE_FACE_NEGATIVE_Y]; break;
   case 4: img=cubeFace[CUBE_FACE_POSITIVE_Z]; break;
   case 5: img=cubeFace[CUBE_FACE_NEGATIVE_Z]; break;
  }
  //glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA16F_ARB,img->getWidth(),img->getHeight(),
  //             0,img->getFormat(),img->getType(),img->getDataFloat());
  glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,img->getWidth(),img->getHeight(),
               0,img->getFormat(),img->getType(),img->getData());

  printf("2D textura z CubeMap textury %u\n",tex_id[ID+i]);
 }

 // CubeMap textura --------------------------------------------------------

 // vygenerovanie textury
 glGenTextures(1,&tex_id[ID+6]);
 tex_count++; // zvysenie poctu textur
 // pripojenie textury
 glBindTexture(GL_TEXTURE_CUBE_MAP,tex_id[ID+6]);

 glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
 glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);

 img=cubeFace[CUBE_FACE_POSITIVE_X];
 glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X,0,GL_RGB,img->getWidth(),img->getHeight(),
              0,img->getFormat(),img->getType(),img->getData());
 img=cubeFace[CUBE_FACE_NEGATIVE_X];
 glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_X,0,GL_RGB,img->getWidth(),img->getHeight(),
              0,img->getFormat(),img->getType(),img->getData());
 img=cubeFace[CUBE_FACE_POSITIVE_Y];
 glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Y,0,GL_RGB,img->getWidth(),img->getHeight(),
              0,img->getFormat(),img->getType(),img->getData());
 img=cubeFace[CUBE_FACE_NEGATIVE_Y];
 glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,0,GL_RGB,img->getWidth(),img->getHeight(),
              0,img->getFormat(),img->getType(),img->getData());
 img=cubeFace[CUBE_FACE_POSITIVE_Z];
 glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_Z,0,GL_RGB,img->getWidth(),img->getHeight(),
              0,img->getFormat(),img->getType(),img->getData());
 img=cubeFace[CUBE_FACE_NEGATIVE_Z];
 glTexImage2D(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z,0,GL_RGB,img->getWidth(),img->getHeight(),
              0,img->getFormat(),img->getType(),img->getData());

 // INFO: identifikator textury
 printf("CubeMap textura: %u\n",tex_id[ID+6]);

 // uvolnenie pamate obsadenej obsahom 6-tich obrazkov
 loopi(6) DELETEP(cubeFace[i]);
 // uvolnenie pamate obsadenej obsahom suboru so 6-timi obrazkami
 DELETEP(cubeMap);

 return(0);
}
